from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.postgres.search import (
    SearchQuery,
    SearchRank,
    SearchVector,
    TrigramSimilarity,
)
from django.db import models
from django.shortcuts import redirect
from django.views.generic.edit import FormView
from transaction.views import TransactionListView

from .forms import SearchForm


class SearchFormView(LoginRequiredMixin, FormView):
    template_name = "search/search.html"
    form_class = SearchForm

    def form_valid(self, form):
        return redirect("search", search=form.cleaned_data.get("search"))


class SearchView(TransactionListView):
    def get_queryset(self):
        self.search = self.kwargs["search"]
        return (
            super()
            .get_queryset()
            .annotate(
                rank=SearchRank(
                    SearchVector("name", weight="A")
                    + SearchVector("description", weight="B")
                    + SearchVector("trader", weight="B"),
                    SearchQuery(self.search, search_type="websearch"),
                ),
                similarity=TrigramSimilarity("name", self.search),
            )
            .filter(models.Q(rank__gte=0.1) | models.Q(similarity__gte=0.3))
            .order_by("-rank", "-date")
        )

    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs) | {"search": self.kwargs["search"]}
